program approx_it
  use sep
  use sep3d_struct_mod
  use amo_wrap_mod
  use amo_interp_mod
  use amo_approx_mod
  real, pointer :: head2(:,:)
  real, allocatable :: headers(:,:)
  type(sep3d) :: input_s
  complex,allocatable :: mod(:,:,:), pvar(:,:,:),data(:,:)
  integer, external :: sreed
  integer :: n1,nh,n52,n(5)
  integer :: nf,nd
  integer :: i,i4,i5,i_f
  real    :: o(5),d(5),eps,hx(2),hy(2)
  character(len=128) :: temp1
  logical :: verb


  call sep_init(SOURCE)
  call sep_begin_prog()
  call from_param("verb",verb,.false.)
  call from_param("eps",eps,.1) 

  !GET PARAMETERS ABOUT REGULAR SPACE

  call from_param("amx_n",n(2))
  call from_param("amx_o",o(2))
  call from_param("amx_d",d(2))
                                                                                
  call from_param("amy_n",n(3))
  call from_param("amy_o",o(3))
  call from_param("amy_d",d(3))
                                                                                
  call from_param("ahx_n",n(4))
  call from_param("ahx_o",o(4))
  call from_param("ahx_d",d(4))
                                                                                
  call from_param("ahy_n",n(5))
  call from_param("ahy_o",o(5))
  call from_param("ahy_d",d(5))
                                                                                
  call from_param("at__n",n(1))
  call from_param("at__o",o(1),0.)
  call from_param("at__d",d(1))

  call init_sep3d(input_s,"SCRATCH","FLOAT","REGULAR",n,o,d)


  !GET PARAMETERS ABOUT DATA
  call from_history("n2",nf)
  call from_history("n1",nd)


  !READ IN HEADERS
  call from_aux("headers","n1",n1)
  call from_aux("headers","n2",nh)

  if(n1/=4) call seperr("expecting n1 of headers to be 4")
  allocate(headers(n1,nh))
  if(4*size(headers)/=sreed("headers",headers,size(headers)*4)) &
    call seperr("trouble reading in headers")
  
  if(nd /= nh) call seperr("expecting number of headers and data to be the same")

  allocate(head2(5,nh))
  do i=1,size(headers,2)
    head2(1,i)=(headers(1,i)-o(2))/d(2)+1.
    head2(2,i)=(headers(2,i)-o(3))/d(3)+1.
    head2(3,i)=(headers(3,i)-o(4))/d(4)+1.
    head2(4,i)=(headers(4,i)-o(5))/d(5)+1.
    head2(5,i)=(-headers(4,i)-o(5))/d(5)+1.
  end do
  deallocate(headers) 


   
  !initialize temp file
  call from_history("in",temp1)
  call parcat(temp1,923)
  call auxinout(temp1)


  !SETUP OUTPUT SPACE
  call sep_put_data_axis_par("out",1,n(2),o(2),d(2),"CMP_X")
  call sep_put_data_axis_par("out",2,n(3),o(3),d(3),"CMP_Y")
  call sep_put_data_axis_par("out",3,n(4),o(4),d(4),"H_X")
  call sep_put_data_axis_par("out",4,1,0.,d(5),"H_Y")
  call sep_put_data_axis_par("out",5,nf,0,1,"FREQ")


  !INIT OPERATORS
  call amo_approx_init(input_s,nf,head2)


  if(.not.init_print_progress(0,nf*10*n(5)*n(4),verb)) &
    call seperr("trouble initialzing verbosity")

  do i_f=1,nf
    !FIRST CREATE REFERENCE  PVAR
    allocate(mod(n(2),n(3),n(4)*n(5)))
    allocate(pvar(n(2),n(3),n(4)*n(5)))
    pvar=1
    call print_progress("Convert kx, ky",n(4)*n(5))
    call to_k(pvar)
    call print_progress("Preconditioning for reference")
    call prec(.false.,pvar,mod)  ! B 1
    write(0,*) "writing1out",sum(pvar),sum(mod)
    deallocate(pvar); allocate(data(nd,1))
    call print_progress("Forward op reference")
    call op  (.false.,mod,data)  ! L B 1
    write(0,*) "writing2out",sum(data),sum(mod)
    call print_progress("Adjoint op reference")
    call op  (.true. ,mod,data)  ! L' L B 1
    write(0,*) "writing3out",sum(data),sum(mod)
    deallocate(data); allocate(pvar(n(2),n(3),n(4)*n(5)))
    call print_progress("Adjoint preconditioning reference")
    call prec(.true.,pvar,mod)  ! B' L' L B 1
    write(0,*) "writin43out",sum(pvar),sum(mod)
    call srite(temp1,pvar,size(pvar)*8)
    write(0,*) "writing out",sum(pvar)
    
    !NOW DATA
    deallocate(pvar); allocate(data(nd,1))
    if(8*size(data)/=sreed("in",data,size(data)*8)) &
      call seperr("trouble reading in data")

    call print_progress("Adjointing data")
    call op  (.true. ,mod,data)  ! L' d
    deallocate(data); allocate(pvar(n(2),n(3),n(4)*n(5)))
    call print_progress("Adjoint preconditioning data")
    call prec(.true.,pvar,mod)  ! B' L' d
    
    call print_progress("Scale model",n(4)*n(5))
    call sseek (temp1,0,0) 
    if(8*size(mod)/=sreed(temp1,mod,size(mod)*8)) &
      call seperr("trouble reading in pvar")
    call sseek (temp1,0,0) 
     
    do i3=1,n(5)*n(4) 
      do i2=1,n(3)
        do i1=1,n(2)
          pvar(i1,i2,i3)=pvar(i1,i2,i3)/(cabs(mod(i1,i2,i3))+eps)
        end do
      end do
    end do

    call print_progress("Forward preconditioning model")
    call prec(.false.,pvar,mod)  !    B S B' L' d

    call print_progress("To space")
    call to_x(mod)

    write(0,*) "writing out",sum(mod)
    call srite("out",mod,size(mod)*8)
    deallocate(pvar,mod)
  end do
  call sep_end_prog()
end program
