import SEP.opt_none
import amo_time
import amo_zero
import amo_prep
import amo_smooth
import amo_interp
import SEP.sort3d
import math


class amo_main(SEP.opt_noen.options):
  """GENERAL AMO"""
  def __init__(self,name="AMO_MAIN"):
    SEP.opt_noen.options.__init__(self,name)
    self.add_doc_params()
  def add_doc_params(self):
    """Parameters for doing amo adjoint"""
    self.add_doc_param("restart",None,"Whether or not we are restarting",required=None)
    self.add_doc_param("debug",None,"Whether to enable python debugging",required=None)
    self.add_doc_param("maxsize",1000,"Maximum memory in megabytes")
    self.add_doc_param("verb","y","Whether or not to be verbose (integer)",required=None)
    self.add_doc_param("print_all",None,"Whether or not to print every command that is run",required=None)

    self.add_doc_param("key_mx","cmp_x","Keyname for CMP position in x")

    self.add_doc_param("key_my","cmp_y","Keyname for CMP position in y")
    self.add_doc_param("key_hx","h_x","Keyname for half offset position in x")
    self.add_doc_param("key_hy","h_y","Keyname for half offset position in x")

    self.add_doc_param("amx_n",doc="Number of samples along midpoint x")
    self.add_doc_param("amy_n",doc="Number of samples along midpoint y")
    self.add_doc_param("ahx_n",doc="Number of samples along half-offset x")
    self.add_doc_param("ahy_n",doc="Number of samples along half-offset y")
                                                                                
    self.add_doc_param("amx_o",doc="First sample along midpoint x")
    self.add_doc_param("amy_o",doc="First sample along midpoint y")
    self.add_doc_param("ahx_o",doc="First sample along half-offset x")
    self.add_doc_param("ahy_o",doc="First sample along half-offset y")
                                                                                
    self.add_doc_param("amx_d",doc="Sampling along midpoint x")
    self.add_doc_param("amy_d",doc="Sampling along midpoint y")
    self.add_doc_param("ahx_d",doc="Sampling along half-offset x")
    self.add_doc_param("ahy_d",doc="Sampling along half-offset y")

    self.add_doc_param("f_max",60.,doc="Maximum frequency")
    self.add_doc_param("t_min_log",.1,doc="Minimum time for logsretch")
                                                                                
    self.add_doc_param("irregular",doc="Irregular SEP3D-dataset (input)")
    self.add_doc_param("regular",doc="Regular SEP dataset (output)")
    self.add_doc_param("vrms",doc="Vrms(t)")
  def prep_run(self,restart=None):
    """Check for parameter errors"""

    if self.logic("verb"):  self.add_param("verb_py",1)
    else: self.add_param("verb_py",None)
    
    for i in ["amx_n","amy_n"]:
      if math.fmod(int(self.param(i)),2) != 0:
        SEP.util.err("All axes need to be even in length %s=%s"%(i,str(self.param(i))))

  def run_job(self,restart,extra=None):
    """How to run the job"""
  def clean_files(self):
    """How I clean"""

class amo_inverse(amo_main):
  """The adjoint operatoion"""
  def __init__(self,name="AMO_INVERSE.main"):
    amo_main.__init__(self,name)
  def add_doc_params(self):
    """Parameters for doing amo adjoint"""
    amo_main.add_doc_params(self)
    self.add_doc_param("niter",10,"Number of iterations")
    self.add_doc_param("eps",doc="Epsilon")
    self.add_doc_param("mod_movie",doc="Base name for model movie (only zero hy)",required=None)



class amo_inverse_eps0(amo_inverse):
  """The adjoint operatoion"""
  def __init__(self,name="AMO_INVERSE.main"):
    amo_inverse.__init__(self,name)
  def add_doc_params(self):
    """Parameters for doing amo adjoint"""
    amo_inverse.add_doc_params(self)
    self.del_par("eps")






class amo_adj_main(amo_main):
  """The adjoint operatoion"""
  def __init__(self,name="AMO_ADJ.main"):
    amo_main.__init__(self,name)

  def run_job(self,restart,extra=None):
    """How to run the job"""
  def clean_files(self):
    """How I clean"""



class amo_sort(SEP.sort3d.sort3d):
  """Sort3d for AMO"""
  def __init__(self,main_par,name="AMO_ADJ.SORT"):
    """Initialze the lloyd operator WEI migration"""
    SEP.sort3d.sort3d.__init__(self,name)
    self.main=main_par
  def add_doc_params(self):
    """Add the documentation for Sort3d need for AMO"""
    SEP.sort3d.sort3d.add_doc_params(self)
    self.del_doc_param("key1");self.del_doc_param("key2")
    self.del_doc_param("key3");self.del_doc_param("key4")
    self.del_doc_param("key5");self.del_doc_param("key6")
    self.del_doc_param("plane"); self.del_doc_param("preserve_nd")
    self.del_doc_param("nkeys");self.del_doc_param("ng")
    self.del_doc_param("dg");self.del_doc_param("og")
    self.del_doc_param("stdin");self.del_doc_param("stdout")
    self.del_doc_param("compress_tr");self.del_doc_param("synch")
    self.del_doc_param("verb");self.del_doc_param("max_size")
  def build_check_params(self):
    """Build commands for Sort3d needed for AMO"""
    self.add_param("compress_tr",0)
    self.add_param("nkeys",4)
    self.add_param("key1",self.main.param("key_mx"))
    self.add_param("key2",self.main.param("key_my"))
    self.add_param("key3",self.main.param("key_hx"))
    self.add_param("key4",self.main.param("key_hy"))
    self.add_param("ng", [int(self.main.param("amx_n")),
    int(self.main.param("amy_n")), int(self.main.param("ahx_n")),
    int(self.main.param("ahy_n"))])
    self.add_param("verb",self.main.param("verb"))

    self.add_param("og", [float(self.main.param("amx_o")),
    float(self.main.param("amy_o")), float(self.main.param("ahx_o")),
    float(self.main.param("ahy_o"))])

    self.add_param("dg", [float(self.main.param("amx_d")),
    float(self.main.param("amy_d")), float(self.main.param("ahx_d")),
    float(self.main.param("ahy_d"))])
    
    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    if not file.headers: SEP.util.err("Headers must exist in the data")
    self.add_param("stdin",fname)
    self.add_param("stdout","%s.sort"%fname)
    SEP.sort3d.sort3d.build_check_params(self)
#  def run_job(self,restart,extra=None):
#       """How to run the job"""
#  def clean_files(self):
#    """How I clean"""
#


class to_freq(amo_prep.to_freq):
  """Convert to zero hy"""
  def __init__(self,main_par,par_par,name="AMO_ADJ.TO_FREQ"):
    """Initialze the Log-stretch and FFT"""
    amo_prep.to_freq.__init__(self,name)
    self.main=main_par
    self.par_par=par_par
  def add_doc_params(self):
    """Add the documentation for AMO TO FREQ"""
    amo_prep.to_freq.add_doc_params(self)
    self.del_doc_param("mx_key");self.del_doc_param("my_key")
    self.del_doc_param("hx_key");self.del_doc_param("hy_key")
    self.del_doc_param("stdin");self.del_doc_param("stdout")
    self.del_doc_param("vrms")
    self.del_doc_param("headers")
    self.del_doc_param("restart")
    self.del_doc_param("t_min_log");self.del_doc_param("f_max")
    self.delete_cl_par_params()
    self.del_doc_param("verb");self.del_doc_param("maxsize")
  def build_check_params(self):
    """Build commands for converting to stretching frequency"""
    self.set_cl_par_params(self.par_par.return_set_params())
    self.add_param("maxsize",self.main.param("maxsize"))
    self.add_param("verb",self.main.param("verb"))

  def prep_run(self,restart=None):
    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    if not file.headers: SEP.util.err("Headers must exist in the data")
    self.add_param("stdin","%s.sort"%fname)
    self.add_param("vrms",self.main.param("vrms"))
    self.add_param("stdout","%s.stretch"%fname)
    self.add_param("headers","%s.headers"%fname)
    self.add_param("mx_key",self.main.param("key_mx"))
    self.add_param("my_key",self.main.param("key_my"))
    self.add_param("hx_key",self.main.param("key_hx"))
    self.add_param("hy_key",self.main.param("key_hy"))
    self.add_param("f_max",self.main.param("f_max"))
    self.add_param("t_min_log",self.main.param("t_min_log"))
    amo_prep.to_freq.prep_run(self,restart=None)
    self.main.add_param("freq_irreg",self.par_files["outtag"])
#  def run_job(self,restart,extra=None):
#    """How to run the job"""
#  def clean_files(self):
#    """How I clean"""



class to_zero(amo_zero.to_zero):
  """Convert to zero hy"""
  def __init__(self,main_par,par_par,name="AMO_ADJ.TO_ZERO"):
    """Initialze the Log-stretch and FFT"""
    amo_zero.to_zero.__init__(self,name)
    self.main=main_par
    self.par_par=par_par
  def add_doc_params(self):
    """Add the documentation for AMO TO FREQ"""
    amo_zero.to_zero.add_doc_params(self)
    self.del_doc_param("stdin");self.del_doc_param("stdout")
    self.del_doc_param("headers")
    self.del_doc_param("restart")
    self.del_doc_param("t_min_log");self.del_doc_param("f_max")
    for v in ["mx","my","hy","hx","t_"]:
      self.del_doc_param("a%s_n"%v)
      self.del_doc_param("a%s_o"%v)
      self.del_doc_param("a%s_d"%v)
    self.del_doc_param("distributed_in")
    self.del_doc_param("collect_out")
    self.delete_cl_par_params()
    self.del_doc_param("maxsie");self.del_doc_param("verb")
  def build_check_params(self):
    """Build commands for converting to stretching frequency"""
    self.set_cl_par_params(self.par_par.return_set_params())
    self.add_param("maxsize",self.main.param("vmaxsize"))
    self.add_param("verb",self.main.param("verb"))
  def prep_run(self,restart=None):

    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    self.add_param("old_file","%s.stretch"%fname)
#    self.add_param("stdin","%s.stretch_reorder"%fname)
    self.add_param("stdin","%s.stretch"%fname)
    self.add_param("headers","%s.headers"%fname)
    self.add_param("stdout","%s.zero"%fname)
    n,o,d,label,unit=file.axis(1)
    self.add_param("at__n",n)
    self.add_param("at__o",o)
    self.add_param("at__d",d)
    for v in ["mx","my","hy","hx"]:
      self.add_param("a%s_n"%v,self.main.param("a%s_n"%v))
      self.add_param("a%s_o"%v,self.main.param("a%s_o"%v))
      self.add_param("a%s_d"%v,self.main.param("a%s_d"%v))

    self.add_param("f_max",self.main.param("f_max"))
    self.add_param("t_min_log",self.main.param("t_min_log"))
    self.add_param("collect_out","y")
    self.add_param("distributed_in","n")
    self.add_param("freq_irreg",self.main.param("freq_irreg"))
    amo_zero.to_zero.prep_run(self,restart=None)
    self.main.add_param("freq_reg",self.par_files["outtag"])
#  def run_job(self,restart,extra=None):
#    """How to run the job"""
#  def clean_files(self):
#    """How I clean"""



class to_time(amo_time.to_time):
  """Convert to zero hy"""
  def __init__(self,main_par,par_par,name="AMO_ADJ.TO_ZERO"):
    """Initialze the Log-stretch and FFT"""
    amo_time.to_time.__init__(self,name)
    self.main=main_par
    self.par_par=par_par
  def add_doc_params(self):
    """Add the documentation for AMO TO FREQ"""
    amo_time.to_time.add_doc_params(self)
    self.del_doc_param("stdin");self.del_doc_param("stdout")
    self.del_doc_param("vrms")
    self.del_doc_param("restart")
    self.del_doc_param("at__n")
    self.del_doc_param("at__o")
    self.del_doc_param("at__d")
    self.del_doc_param("t_min_log")
    self.del_doc_param("f_max")
    self.del_doc_param("distributed_in")
    self.delete_cl_par_params()
    self.del_doc_param("maxsize");self.del_doc_param("verb")
  def build_check_params(self):
    """Build commands for converting to stretching frequency"""
    self.set_cl_par_params(self.par_par.return_set_params())
    self.add_param("maxsize",self.main.param("maxsize"))
    self.add_param("verb",self.main.param("verb"))
  def prep_run(self,restart=None):

    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    self.add_param("vrms",self.main.param("vrms"))
    self.add_param("f_max",self.main.param("f_max"))
    self.add_param("t_min_log",self.main.param("t_min_log"))
    
    self.add_param("old_file","%s.zero"%fname)
#    self.add_param("stdin","%s.zero.reoder"%fname)
    self.add_param("stdin","%s.zero"%fname)
    self.add_param("headers","%s.headers"%fname)
    self.add_param("stdout",self.main.param("regular"))
    n,o,d,label,unit=file.axis(1)
    self.add_param("at__n",n)
    self.add_param("at__o",o)
    self.add_param("at__d",d)

    self.add_param("distributed_in","n")
    self.add_param("freq_reg",self.main.param("freq_reg"))
    amo_time.to_time.prep_run(self,restart)
#
#  def run_job(self,restart,extra=None):
#    """How to run the job"""
#  def clean_files(self):
#    """How I clean"""

class interp_op(amo_interp.op):
  """Smooth over offset cubes"""
  def __init__(self,main_par,par_par,name="AMO_SMOOTH"):
    """Initialze the Log-stretch and FFT"""
    amo_interp.op.__init__(self,name)
    self.main=main_par
    self.par_par=par_par
  def add_doc_params(self):
    """Add the documentation for AMO TO FREQ"""
    amo_interp.op.add_doc_params(self)
    self.del_doc_param("headers")
    self.del_doc_param("restart")
    self.del_doc_param("t_min_log");self.del_doc_param("f_max")
    for v in ["mx","my","hy","hx","t_"]:
      self.del_doc_param("a%s_n"%v)
      self.del_doc_param("a%s_o"%v)
      self.del_doc_param("a%s_d"%v)
    self.delete_cl_par_params()
    self.del_doc_param("maxsize");self.del_doc_param("verb")
    self.del_doc_param("spread")
  def build_check_params(self):
    """Build commands for converting to stretching frequency"""
    self.set_cl_par_params(self.par_par.return_set_params())
    self.add_param("maxsize",self.main.param("maxsize"))
    self.add_param("verb",self.main.param("verb"))
    self.add_param("spread","y")
  def prep_run(self,restart=None):

    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    self.add_param("headers","%s.headers"%fname)
    n,o,d,label,unit=file.axis(1)
    self.add_param("at__n",n)
    self.add_param("at__o",o)
    self.add_param("at__d",d)
    for v in ["mx","my","hy","hx"]:
      self.add_param("a%s_n"%v,self.main.param("a%s_n"%v))
      self.add_param("a%s_o"%v,self.main.param("a%s_o"%v))
      self.add_param("a%s_d"%v,self.main.param("a%s_d"%v))

    self.add_param("f_max",self.main.param("f_max"))
    self.add_param("t_min_log",self.main.param("t_min_log"))
    amo_interp.op.prep_run(self,restart)
#  def run_job(self,restart,extra=None):
#    """How to run the job"""
#  def clean_files(self):
#    """How I clean"""


class smooth_op(amo_smooth.op):
  """Smooth over offset cubes"""
  def __init__(self,main_par,par_par,name="AMO_SMOOTH"):
    """Initialze the Log-stretch and FFT"""
    amo_smooth.op.__init__(self,name)
    self.main=main_par
    self.par_par=par_par
  def add_doc_params(self):
    """Add the documentation for AMO TO FREQ"""
    amo_smooth.op.add_doc_params(self)
    self.del_doc_param("headers")
    self.del_doc_param("restart")
    self.del_doc_param("spread")
    self.del_doc_param("t_min_log");self.del_doc_param("f_max")
    for v in ["mx","my","hy","hx","t_"]:
      self.del_doc_param("a%s_n"%v)
      self.del_doc_param("a%s_o"%v)
      self.del_doc_param("a%s_d"%v)
    self.delete_cl_par_params()
    self.del_doc_param("maxsize");self.del_doc_param("verb")
  def build_check_params(self):
    """Build commands for converting to stretching frequency"""
    self.set_cl_par_params(self.par_par.return_set_params())
  def prep_run(self,restart=None):

    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    self.add_param("headers","%s.headers"%fname)
    self.add_param("verb",self.main.param("verb"))
    self.add_param("spread","y")
    n,o,d,label,unit=file.axis(1)
    self.add_param("at__n",n)
    self.add_param("at__o",o)
    self.add_param("at__d",d)
    for v in ["mx","my","hy","hx"]:
      self.add_param("a%s_n"%v,self.main.param("a%s_n"%v))
      self.add_param("a%s_o"%v,self.main.param("a%s_o"%v))
      self.add_param("a%s_d"%v,self.main.param("a%s_d"%v))

    self.add_param("f_max",self.main.param("f_max"))
    self.add_param("t_min_log",self.main.param("t_min_log"))
    amo_smooth.op.prep_run(self,restart)


