import SEP.opt_none
import amo_adj
import SEP.sort3d
import amo_approx_patch
import amo_inv_patch
import math


class amo_main(SEP.opt_none.options):
  """GENERAL AMO"""
  def __init__(self,name="AMO_MAIN"):
    SEP.opt_none.options.__init__(self,name)
    self.add_doc_params()
  def add_doc_params(self):
    """Parameters for doing amo adjoint"""
    self.add_doc_param("restart",None,"Whether or not we are restarting",required=None)
    self.add_doc_param("debug",None,"Whether to enable python debugging",required=None)
    self.add_doc_param("maxsize",1000,"Maximum memory in megabytes")
    self.add_doc_param("verb","y","Whether or not to be verbose (integer)",required=None)
    self.add_doc_param("print_all",None,"Whether or not to print every command that is run",required=None)

    self.add_doc_param("key_mx","cmp_x","Keyname for CMP position in x")

    self.add_doc_param("key_my","cmp_y","Keyname for CMP position in y")
    self.add_doc_param("key_hx","h_x","Keyname for half offset position in x")
    self.add_doc_param("key_hy","h_y","Keyname for half offset position in x")

    self.add_doc_param("amx_n",doc="Number of samples along midpoint x")
    self.add_doc_param("amy_n",doc="Number of samples along midpoint y")
    self.add_doc_param("ahx_n",doc="Number of samples along half-offset x")
    self.add_doc_param("ahy_n",doc="Number of samples along half-offset y")
                                                                                
    self.add_doc_param("amx_o",doc="First sample along midpoint x")
    self.add_doc_param("amy_o",doc="First sample along midpoint y")
    self.add_doc_param("ahx_o",doc="First sample along half-offset x")
    self.add_doc_param("ahy_o",doc="First sample along half-offset y")
                                                                                
    self.add_doc_param("amx_d",doc="Sampling along midpoint x")
    self.add_doc_param("amy_d",doc="Sampling along midpoint y")
    self.add_doc_param("ahx_d",doc="Sampling along half-offset x")
    self.add_doc_param("ahy_d",doc="Sampling along half-offset x")

    self.add_doc_param("f_max",60.,doc="Maximum frequency")
    self.add_doc_param("t_min_log",.1,doc="Minimum time for logsretch")
                                                                                
    self.add_doc_param("irregular",doc="Irregular SEP3D-dataset (input)")
    self.add_doc_param("regular",doc="Regular SEP dataset (output)")
    self.add_doc_param("vrms",doc="Vrms(t)")
  def prep_run(self,restart=None):
    """Check for parameter errors"""

    if self.logic("verb"):  self.add_param("verb_py",1)
    else: self.add_param("verb_py",None)
    
    for i in ["amx_n","amy_n"]:
      if math.fmod(int(self.param(i)),2) != 0:
        SEP.util.err("All axes need to be even in length %s=%s"%(i,str(self.param(i))))

class amo_adj_main(amo_main):
  """The adjoint operatoion"""
  def __init__(self,name="AMO_ADJ.main"):
    amo_main.__init__(self,name)

  def prep_run(self,restart,extra=None):
    """How to run the job"""
  def clean_files(self):
    """How I clean"""

class amo_sort(SEP.sort3d.sort3d):
  """Sort3d for AMO"""
  def __init__(self,main_par,name="AMO_ADJ.SORT"):
    """Initialze the lloyd operator WEI migration"""
    SEP.sort3d.sort3d.__init__(self,name)
    self.main=main_par
  def add_doc_params(self):
    """Add the documentation for Sort3d need for AMO"""
    SEP.sort3d.sort3d.add_doc_params(self)
    self.del_doc_param("key1");self.del_doc_param("key2")
    self.del_doc_param("key3");self.del_doc_param("key4")
    self.del_doc_param("key5");self.del_doc_param("key6")
    self.del_doc_param("plane"); self.del_doc_param("preserve_nd")
    self.del_doc_param("nkeys");self.del_doc_param("ng")
    self.del_doc_param("dg");self.del_doc_param("og")
    self.del_doc_param("stdin");self.del_doc_param("stdout")
    self.del_doc_param("compress_tr");self.del_doc_param("synch")
    self.del_doc_param("verb");self.del_doc_param("max_size")
  def build_check_params(self):
    """Build commands for Sort3d needed for AMO"""
    self.add_param("compress_tr",0)
    self.add_param("nkeys",4)
    self.add_param("key1",self.main.param("key_mx"))
    self.add_param("key2",self.main.param("key_my"))
    self.add_param("key3",self.main.param("key_hx"))
    self.add_param("key4",self.main.param("key_hy"))
    self.add_param("ng", [int(self.main.param("amx_n")),
    int(self.main.param("amy_n")), int(self.main.param("ahx_n")),
    int(self.main.param("ahy_n"))])
    self.add_param("debug","1")
    self.add_param("verb","1")

    self.add_param("og", [float(self.main.param("amx_o")),
    float(self.main.param("amy_o")), float(self.main.param("ahx_o")),
    float(self.main.param("ahy_o"))])

    self.add_param("dg", [float(self.main.param("amx_d")),
    float(self.main.param("amy_d")), float(self.main.param("ahx_d")),
    float(self.main.param("ahy_d"))])
    
    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    if not file.headers: SEP.util.err("Headers must exist in the data")
    self.add_param("stdin",fname)
    self.add_param("stdout","%s.sort"%fname)

  def prep_run(self,restart=None):
    """AAA""" 

class approx_patch(amo_approx_patch.approx):
  """Convert to zero hy"""
  def __init__(self,main_par,par_par,name="AMO_ADJ.TO_FREQ"):
    """Initialze the Log-stretch and FFT"""
    amo_approx_patch.approx.__init__(self,name)
    self.main=main_par
    self.par_par=par_par
  def add_doc_params(self):
    """Add the documentation for AMO TO FREQ"""
    amo_approx_patch.approx.add_doc_params(self)
    self.del_doc_param("mx_key");self.del_doc_param("my_key")
    self.del_doc_param("hx_key");self.del_doc_param("hy_key")
    self.del_doc_param("stdin");self.del_doc_param("stdout")
    self.del_doc_param("vrms")
    self.del_doc_param("restart")
    self.del_doc_param("t_min_log");self.del_doc_param("f_max")
    self.delete_cl_par_params()
    self.del_doc_param("verb");self.del_doc_param("maxsize")
  def build_check_params(self):
    """Build commands for converting to stretching frequency"""
    self.set_cl_par_params(self.par_par.return_set_params())
    self.add_param("maxsize",self.main.param("maxsize"))
    self.add_param("verb",self.main.param("verb"))

  def prep_run(self,restart=None):
    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    if not file.headers: SEP.util.err("Headers must exist in the data")
    self.add_param("stdin","%s.sort"%fname)
    self.add_param("vrms",self.main.param("vrms"))
    self.add_param("stdout",self.main.param("regular"))
    self.add_param("headers","%s.headers"%fname)
    self.add_param("mx_key",self.main.param("key_mx"))
    self.add_param("my_key",self.main.param("key_my"))
    self.add_param("hx_key",self.main.param("key_hx"))
    self.add_param("hy_key",self.main.param("key_hy"))
    self.add_param("f_max",self.main.param("f_max"))
    self.add_param("t_min_log",self.main.param("t_min_log"))
    amo_apporx_patch.aporx.prep_run(self,restart)
    self.main.add_param("freq_irreg",self.par_files["outtag"])



class inv_patch(amo_inv_patch.inv):
  """Convert to zero hy"""
  def __init__(self,main_par,par_par,name="AMO_ADJ.TO_FREQ"):
    """Initialze the Log-stretch and FFT"""
    amo_inv_patch.inv.__init__(self,name)
    self.main=main_par
    self.par_par=par_par
  def add_doc_params(self):
    """Add the documentation for AMO TO FREQ"""
    amo_inv_patch.inv.add_doc_params(self)
    self.del_doc_param("mx_key");self.del_doc_param("my_key")
    self.del_doc_param("hx_key");self.del_doc_param("hy_key")
    self.del_doc_param("stdin");self.del_doc_param("stdout")
    self.del_doc_param("vrms")
    self.del_doc_param("restart")
    self.del_doc_param("t_min_log");self.del_doc_param("f_max")
    self.delete_cl_par_params()
    self.del_doc_param("verb");self.del_doc_param("maxsize")
  def build_check_params(self):
    """Build commands for converting to stretching frequency"""
    self.set_cl_par_params(self.par_par.return_set_params())
    self.add_param("maxsize",self.main.param("maxsize"))
    self.add_param("verb",self.main.param("verb"))

  def prep_run(self,restart=None):
    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    if not file.headers: SEP.util.err("Headers must exist in the data")
    self.add_param("stdin","%s.sort"%fname)
    self.add_param("vrms",self.main.param("vrms"))
    self.add_param("stdout",self.main.param("regular"))
    self.add_param("headers","%s.headers"%fname)
    self.add_param("mx_key",self.main.param("key_mx"))
    self.add_param("my_key",self.main.param("key_my"))
    self.add_param("hx_key",self.main.param("key_hx"))
    self.add_param("hy_key",self.main.param("key_hy"))
    self.add_param("f_max",self.main.param("f_max"))
    self.add_param("t_min_log",self.main.param("t_min_log"))
    amo_inv_patch.inv.prep_run(self,restart)
    self.main.add_param("freq_irreg",self.par_files["outtag"])




class adj(amo_adj.adj):
  """Convert to zero hy"""
  def __init__(self,main_par,par_par,name="AMO_ADJ.TO_FREQ"):
    """Initialze the Log-stretch and FFT"""
    amo_adj.adj.__init__(self,name)
    self.main=main_par
    self.par_par=par_par
  def add_doc_params(self):
    """Add the documentation for AMO TO FREQ"""
    amo_adj.adj.add_doc_params(self)
    self.del_doc_param("mx_key");self.del_doc_param("my_key")
    self.del_doc_param("hx_key");self.del_doc_param("hy_key")
    self.del_doc_param("stdin");self.del_doc_param("stdout")
    self.del_doc_param("vrms")
    self.del_doc_param("restart")
    self.del_doc_param("t_min_log");self.del_doc_param("f_max")
    self.delete_cl_par_params()
    self.del_doc_param("verb");self.del_doc_param("maxsize")
  def build_check_params(self):
    """Build commands for converting to stretching frequency"""
    self.set_cl_par_params(self.par_par.return_set_params())
    self.add_param("maxsize",self.main.param("maxsize"))
    self.add_param("verb",self.main.param("verb"))

  def prep_run(self,restart=None):
    fname=self.main.param("irregular")
    file=SEP.sepfile.sep_file(fname)
    
    if not file.headers: SEP.util.err("Headers must exist in the data")
    self.add_param("stdin","%s.sort"%fname)
    self.add_param("vrms",self.main.param("vrms"))
    self.add_param("stdout",self.main.param("regular"))
    self.add_param("headers","%s.headers"%fname)
    self.add_param("mx_key",self.main.param("key_mx"))
    self.add_param("my_key",self.main.param("key_my"))
    self.add_param("hx_key",self.main.param("key_hx"))
    self.add_param("hy_key",self.main.param("key_hy"))
    self.add_param("f_max",self.main.param("f_max"))
    self.add_param("t_min_log",self.main.param("t_min_log"))
    amo_adj.adj.prep_run(self,restart)
    self.main.add_param("freq_irreg",self.par_files["outtag"])


