module amo_inv_mod
  use sep3d_struct_mod
  use amo_wrap_mod
  use amo_smooth_mod
  use amo_interp_mod
  use polydiv_func_mod
  use oc_solverpre_mod
  use oc_cgstep_mod
  use oc_combine_mod
  use sep
  use verb_mod
  use random
  implicit none
  real, private, save   :: hx_dist
  real, private,save    :: o(5),d(5)
  integer, private,save :: n(5),nf,nt,niter
  type(sep3d),private,save           :: irreg_s,input_s,output_s
  real,private,save                  :: eps 
  logical,private,save               :: normalize
  character(len=128),private,save :: temp
  

  contains
  logical function amo_inv_init(d_s,freq_i,freq_reg,init_amo)
    type(sep3d) :: d_s,freq_reg,freq_i
    integer :: nout(5),i,i_f
    real  :: oout(5),dout(5)
    logical :: init_amo
    amo_inv_init=.false.

    if(init_amo) call amo_get_n1_size(d_s,nf,.true.,0)
    irreg_s=freq_i
    nt=max(100,irreg_s%n(1))
    o=0;d=1;n(5)=irreg_s%n(2)
    n(1:4)=d_s%n(2:5)
    o(1:4)=d_s%o(2:5)
    d(1:4)=d_s%d(2:5)
    oout=o; dout=d; nout=n
!    oout(4)=0.; nout(4)=1
    call from_param("ahx_n_o",nout(3),nout(3))
    call from_param("ahx_o_o",oout(3),oout(3))
    call from_param("ahx_d_o",dout(3),dout(3))
    call from_param("hx_dist",hx_dist,3*d(3))
    call init_sep3d(input_s,"OUTPUT","COMPLEX","REGULAR",n,o,d)
    call init_sep3d(freq_reg,"OUTPUT","COMPLEX","REGULAR",nout,oout,dout)
    output_s=freq_reg
    call from_param("niter",niter,3)
    call from_param("eps",eps,10.)

    amo_inv_init=.true.
  end function

                                                                                
 integer function smooth_interp_op(adj,add,mod_tag,dat_tag,op1,op2,op3,op4,op5,op6,op7,op8,op9)
  logical,            intent(in) :: adj,add
  character(len=128), intent(in) :: mod_tag,dat_tag
  integer,external,optional :: op1,op2,op3,op4,op5,op6,op7,op8,op9
                                                                                
   call oc_chain2(interp_op,smooth_op,adj,add,mod_tag,dat_tag,temp,op1,op2,op3,op4,op5,op6,op7,op8,op9)

   smooth_interp_op=0
  end function



  subroutine amo_approx(intag,outtag,locs,verb)
    character(len=*)    :: intag,outtag
    real                :: locs(:,:)
    real,pointer    ::  headers(:,:),tref(:,:)
    complex,allocatable ::  mod(:,:,:,:,:),dat(:,:),ref(:,:,:,:,:),tempr(:,:),out(:,:,:,:,:),fref(:,:)
    integer             :: ndone,nblock,iyb,ibeg,i,ierr
    logical             :: verb
    character(len=128)  :: xtag,ytag,ptag,name


    allocate(headers(5,size(locs,2)))
    do i=1,size(headers,2)
      headers(1,i)=(locs(1,i)-o(1))/d(1)+1.
      headers(2,i)=(locs(2,i)-o(2))/d(2)+1.
      headers(3,i)=(locs(3,i)-o(3))/d(3)+1.
      headers(4,i)=(locs(4,i)-o(4))/d(4)+1.
      headers(5,i)=(-locs(4,i)-o(4))/d(4)+1.
    end do

    call  interp_init(headers,irreg_s%n,n )
    call  amo_smooth_init(n)
    xtag=trim(outtag)
    ytag=trim(intag)
    call oc_solverpre_init(niter,eps,100*1000*1000,verb=.true.)
    call  oc_solverpre(interp_op,smooth_op,oc_cgstep, xtag,ytag,n)
    deallocate(headers)
    call aux_unlink(temp)
    call aux_unlink(ptag)
  end subroutine

  subroutine amo_inv(intag,outtag,locs,verb)
    character(len=*)    :: intag,outtag
    real                :: locs(:,:)
    real,pointer    ::  headers(:,:),tref(:,:)
    complex,allocatable ::  mod(:,:,:,:,:),dat(:,:),ref(:,:,:,:,:),tempr(:,:),out(:,:,:,:,:),fref(:,:)
    integer             :: ndone,nblock,iyb,ibeg,i,ierr
    logical             :: verb
    character(len=128)  :: xtag,ytag,ptag,name


    allocate(headers(5,size(locs,2)))
    do i=1,size(headers,2)
      headers(1,i)=(locs(1,i)-o(1))/d(1)+1.
      headers(2,i)=(locs(2,i)-o(2))/d(2)+1.
      headers(3,i)=(locs(3,i)-o(3))/d(3)+1.
      headers(4,i)=(locs(4,i)-o(4))/d(4)+1.
      headers(5,i)=(-locs(4,i)-o(4))/d(4)+1.
    end do

    call  interp_init(headers,irreg_s%n,n )
    call  amo_smooth_init(n)
    xtag=trim(outtag)
    ytag=trim(intag)
    call oc_solverpre_init(niter,eps,100*1000*1000,verb=.true.)
    call  oc_solverpre(interp_op,smooth_op,oc_cgstep, xtag,ytag,n)
    deallocate(headers)
  end subroutine

  subroutine amo_reg_to_zero(intag,outtag,verb)
    character(len=*)    :: intag,outtag
    logical :: verb
    integer, external :: sreed,srite
    integer :: fw(5),nw(5),jw(5)
    integer :: nfreq,ifreq,i,i3,i4
    complex, allocatable :: mod(:,:,:),dat(:,:,:),tempr(:,:)
    real :: hx(2),hy(2)
                                                                                
    nfreq=n(5)
                                                                                
    allocate(mod(n(1),n(2),n(3)*n(4)),dat(n(1),n(2),n(3)))
    allocate(tempr(n(1),n(2)))
    do ifreq=1,nfreq
write(0,*) "BBB a",ifreq,n
                                                                                

      !transform to kx,ky
      do i=1,n(4)*n(3)
        if(8*size(tempr)/=sreed(intag,tempr,size(tempr)*8)) &
          call seperr("trouble reading in mod")
        call amo_hx_transform(hx,hy,tempr,.true.,.false.,.false.,.false.)
        mod(:,:,i)=tempr
      end do
                                                                                  
      dat=0
      hy(2)=0
      i=0
      do i4=1,n(4)
        do i3=1,n(3)
          i=i+1
          hx=(i3-1)*d(3)+o(3)
          hy(1)=(i4-1)*d(4)+o(4)
          tempr=mod(:,:,i)
          call amo_hx_transform(hx,hy,tempr,.false.,.false.,.true.,.false.)
          dat(:,:,i3)=tempr+dat(:,:,i3)
        end do
      end do
      !smooth
      !transfer back to mx,my
      do i=1,n(3)
        tempr=dat(:,:,i)
        call amo_hx_transform(hx,hy,tempr,.false.,.true.,.false.,.false.)
        if(8*size(tempr)/=srite(outtag,tempr,8*size(tempr))) &
          call seperr("trouble writing out dat")
      end do
    end do
    call sseek(intag,0,0)
    call sseek(outtag,0,0)
    deallocate(dat,mod,tempr)
end subroutine
end module
