import SEP.sepfile
import SEP.util
import SEP.spawn
import types,math
import string
import SEP.pj_base
import SEP.pf_copy
import SEP.pf_irreg
import SEP.pf_patch
import SEP.paths
__author__ = "Robert G. Clapp"
__email__ = "bob@sep.stanford.edu"
__version = "2005.03.26"


class inv(SEP.pj_base.par_job):
  def __init__(self,name="AMO_to_zero"):
    """Initialize the AMO  program"""
    SEP.pj_base.par_job.__init__(self,name)
    self.add_doc_params()
 
  def add_doc_params(self):
    """Parameters for AMO_to_freq"""
    SEP.pj_base.par_job.add_doc_params(self)

    self.add_doc_param("stdin",doc="Input file")
    self.add_doc_param("stdout",doc="Output file")

    self.add_doc_param("maxsize",1000,doc="Maximum amount of memory to use")

    self.add_doc_param("mx_key","cmp_x",doc="Key describing cmp x")
    self.add_doc_param("my_key","cmp_y",doc="Key describing cmp y")
    self.add_doc_param("hx_key","h_x",doc="Key describing h x")
    self.add_doc_param("hy_key","h_y",doc="Key describing h y")
    self.add_doc_param("nblock",doc="Number of blocks",required=None)
    self.add_doc_param("vrms",doc="Vrms(t)")
    self.add_doc_param("noverlap",[6,6],doc="Half the overlap")
    self.add_doc_param("niter",3.,doc="Number of iterations")
    self.add_doc_param("eps",10.,doc="Espilon")
    self.add_doc_param("f_max",60.,doc="Maximum frequency")
    self.add_doc_param("t_min_log",.2,doc="Minimum time for logstrech")
    self.add_doc_param("verb","y",doc="Whether or not to be verbose")
    self.param_list=["mx_key","my_key","hx_key","hy_key","vrms","verb"]
    self.param_list.extend(["t_min_log","f_max","eps","niter"])
    self.delete_object_par_params()


  def prep_run(self,restart=None):  
    """Check to make sure the parameters make sense"""

    restart=self.main.param("restart")

    ins=SEP.sepfile.sep_file(self.param("stdin"))

    nt=int(ins.axis(1)[0])
    maxsize=int(self.param("maxsize"))*1000*1000/(30*int(nt))

    #get the section parameters
    self.add_param("program","AMO_inv_patch.x")

    self.add_param("pverb",self.param("verb") )
    self.par_files={}

    nblock=self.param("nblock",error=1)
    noverlap=self.param("noverlap")
    print "AAA",nblock,noverlap
    if type(noverlap) != types.ListType:
       SEP.util.err("novlerap should be a list of size 2")
    if type(nblock) != types.ListType:
       SEP.util.err("nblock should be a list of size 2")
    if len(nblock) != 2: SEP.util.err("Expecting nblock 2 be length 2")
    if len(noverlap) != 2: SEP.util.err("Expecting noverlap 2 be length 2")
    nb=[]
    ntot=1
    for n in nblock:
      n=int(n)
      ntot=ntot*n
      nb.append(n)

    self.par_files["intag"]=SEP.pf_irreg.parfile(name=self.param("stdin"), 
      dff_axis=[3,4], tag="<",usage="INPUT",njobs=ntot,noverlap=noverlap,
      restart=restart,nblock=nb)

    self.par_files["outtag"]=SEP.pf_patch.parfile(name=self.param("stdout"), 
      dff_axis=[2,3], tag=">",usage="OUTPUT",njobs=ntot,
      restart=restart,nblock=nb,noverlap=noverlap)

    self.par_files["vrms"]=SEP.pf_copy.parfile(name=self.param("vrms"), 
      tag="vel=",usage="INPUT",njobs=ntot,
      restart=restart,nblock=nblock)

    self.add_param("files",self.par_files)
    order=[]
    for i in range(ntot): order.append(str(i))
    self.add_param("sect_pars",SEP.pj_func.simple_parameter_list(ntot))
    self.add_param("order",order)
    self.add_param("global_pars",self.return_set_params(self.param_list))
    SEP.pj_base.par_job.prep_run(self,restart)
    
