module amo_nmo_mod
  use sep3d_struct_mod
  implicit none 
  type(sep3d), private,save :: d_s,s_s
  real, private, pointer :: slow(:,:,:),off(:),slow1(:)
  integer, private, save :: naxes,nkeys,nelem,axis(10)

contains

  subroutine amo_nmo_init(data_s,stag,nsz)
    type(sep3d)  :: data_s,slow_s
    character(len=128) :: stag
    real, pointer :: s1(:,:),s2(:,:),m1(:),m2(:)
    integer :: nk,nax,i3,i2,n,nx,ny,nsz,i1
    real :: fact,sz1,sz2
    
    d_s=data_s 
    call init_sep3d(stag,slow_s,"INPUT")
    call init_sep3d(slow_s,s_s,"SCRATCH")
    n=ceiling(real(slow_s%d(1)*slow_s%n(1))/.03)
    s_s%n(1)=n;s_s%o(1)=0.; s_s%d(1)=.03

    sz1=n*slow_s%n(2)*slow_s%n(3)
    if(sz1 > nsz*1000*1000) then
      fact=sz1/real(nsz)/1000./1000.
      s_s%d(2)=slow_s%d(2)/fact 
      s_s%n(2)=ceiling(slow_s%n(2)*fact)
    end if
    call sep3d_set_sep3d(s_s)
    

    allocate(s1(slow_s%n(1),slow_s%n(2)))
    allocate(s2(n,slow_s%n(2)))
    allocate(m1(slow_s%n(2)),m2(s_s%n(2)))
    allocate(slow(s_s%n(1),s_s%n(2),s_s%n(3)))
    allocate(slow1(data_s%n(1)))
    allocate(off(1))
    do i3=1,s_s%n(3)
      call sreed(stag,s1,size(s1)*4)
      do i2=1,s_s%n(2)
        call interp(slow_s%n(1),slow_s%o(1),slow_s%d(1),s_s%n(1),s_s%o(1),s_s%d(1),s1(:,i2),s2(:,i2))
      end do
      do i1=1,s_s%n(1)
        m1=s2(i1,:)
        call interp(slow_s%n(2),slow_s%o(2),slow_s%d(2),s_s%n(2),s_s%o(2),s_s%d(2),m1,m2)
        slow(i1,:,i3)=m2 
      end do
    end do
    call sseek(stag,0,0)
    deallocate(s1,s2,m1,m2)
  end subroutine

  subroutine deallocate_slow()
    deallocate(slow)
  end subroutine

  subroutine nmoit(input,output,locx,locy,offset)
    real :: input(:,:),output(:,:),locx(:),locy(:),offset(:)
    real,allocatable :: in(:),out(:)
    integer :: i2,ix,iy
    
    allocate(in(size(input,1)),out(size(input,1)))
    do i2=1,size(offset)
      ix=min(max(1,nint((locx(i2)-s_s%o(2))/s_s%d(2)+1.)),s_s%n(2))
      iy=min(max(1,nint((locy(i2)-s_s%o(3))/s_s%d(3)+1.)),s_s%n(3))
      call interp(s_s%n(1),s_s%o(1),s_s%d(1),d_s%n(1),d_s%o(1),d_s%d(1),slow(:,ix,iy),slow1)
      slow1=1./slow1
      in=input(:,i2)
      call nmo_linear(.true.,slow1,offset(i2),out,in)
      output(:,i2)=out
    end do
    deallocate(in,out)
  end subroutine

  subroutine undo_nmoit(input,output,locx,locy,offset)
    real :: input(:,:),output(:,:),locx(:),locy(:),offset(:)
    integer :: i2,ix,iy
    
    do i2=1,size(offset)
      off=offset(i2)
      ix=min(max(1,nint((locx(i2)-s_s%o(2))/s_s%d(2)+1.)),s_s%n(2))
      iy=min(max(1,nint((locy(i2)-s_s%o(3))/s_s%d(3)+1.)),s_s%n(3))
      call interp(s_s%n(1),s_s%o(1),s_s%d(1),d_s%n(1),d_s%o(1),d_s%d(1),slow(:,ix,iy),slow1)
      slow1=1./slow1
      call nmo_linear(.false.,slow1,offset(i2),input(:,i2),output(:,i2))
    end do
  end subroutine
subroutine interp(nin,oin,din,nout,oout,dout,input,output)
  integer :: nin,nout
  real    :: input(nin),output(nout)
  real    :: oin,din,oout,dout
  integer :: i1,bi_in,ia
  real    :: fract,loc_out,loc_in
  i1=1
  if (oout<oin) then
!extend the velocity up
    loc_out=oout
    i1=1
    do while (loc_out < oin .and. i1 <= nout)
      output(i1)=input(1)
      i1=               i1+1
      loc_out=          loc_out+dout
    end do
  else
    loc_out=oout
  end if
  bi_in=1
  loc_in=oin
  do while (bi_in<nin .and. i1<= nout)
    if (loc_in<=loc_out .and. loc_in+din > loc_out) then
      fract=(loc_out-loc_in)/din
      output(i1)=input(bi_in)*(1.-fract)+ input(bi_in+1)*fract
      i1=               i1+1
      loc_out=          loc_out+dout
    else
      bi_in=            bi_in+1
      loc_in=           loc_in+din
    end if
  end do
  if (i1 <= nout) then
    output(i1:nout)=input(nin) !extend the velocity down
  end if
end subroutine

subroutine nmo_linear( adj,  slow,    x,  zz,  tt )
  logical :: adj 
  integer  it, iz,i_f,n
  real  xs, t , z,           slow(:), x, t0, dt, zz(:), tt(:), wt,ft

  n =size(tt)
  t0=d_s%o(1)
  dt=d_s%d(1)
  if(.not. adj) then
    tt=0.
    do iz= 1, n   
      z = t0 + dt*(iz-1)
      xs = x * slow(iz)
      t = sqrt ( z * z + xs * xs) + 1.e-20
      ft = 1  +  (t - t0) / dt
      it = ft 
      ft = ft - it
      if( it < n ) then
        tt(it) = tt(it) + zz(iz) *(1.-ft)
        tt(it+1) = tt(it+1) + zz(iz) * ft
      end if
    end do 
  else
    zz=0.
    do iz= 1, n   
      z = t0 + dt*(iz-1)
      xs = x * slow(iz)
      t = sqrt ( z * z + xs * xs) + 1.e-20
      ft = 1  +  (t - t0) / dt
      it = ft 
      ft = ft - it
      if( it < n ) then
        zz(iz) = zz(iz) + tt(it) *(1.-ft)+tt(it+1)*ft
      end if
    end do 
  end if
end subroutine

end module
