import SEP.sepfile
import SEP.util
import SEP.spawn
import types,math
import string
import SEP.pj_base
import SEP.pf_copy
import SEP.pf_irreg
import SEP.pf_split
import SEP.paths
__author__ = "Robert G. Clapp"
__email__ = "bob@sep.stanford.edu"
__version = "2005.03.18"


class  to_freq(SEP.pj_base.par_job):
  def __init__(self,name="AMO_to_zero"):
    """Initialize the AMO  program"""
    SEP.pj_base.par_job.__init__(self,name)
    self.add_doc_params()
 
  def add_doc_params(self):
    """Parameters for AMO_to_freq"""
    SEP.pj_base.par_job.add_doc_params(self)

    self.add_doc_param("stdin",doc="Input file")
    self.add_doc_param("headers",doc="Input header file")
    self.add_doc_param("stdout",doc="Output file")

    self.add_doc_param("maxsize",1000,doc="Maximum amount of memory to use")

    self.add_doc_param("mx_key","cmp_x",doc="Key describing cmp x")
    self.add_doc_param("my_key","cmp_y",doc="Key describing cmp y")
    self.add_doc_param("hx_key","h_x",doc="Key describing h x")
    self.add_doc_param("hy_key","h_y",doc="Key describing h y")
    self.add_doc_param("nblock",doc="Number of blocks to break the cmp_x axis along")
    self.add_doc_param("vrms",doc="Vrms(t)")
    self.add_doc_param("f_max",60.,doc="Maximum frequency")
    self.add_doc_param("t_min_log",.2,doc="Minimum time for logstrech")
    self.add_doc_param("verb","y",doc="Whether or not to be verbose")
    self.add_doc_param("collect_out","y","Whether or not the output should be collected")
    self.param_list=["mx_key","my_key","hx_key","hy_key","vrms","verb","t_min_log","f_max"]
    self.delete_object_par_params()


  def prep_run(self,restart=None):  
    """Check to make sure the parameters make sense"""

    restart=self.main.param("restart")

    ins=SEP.sepfile.sep_file(self.param("stdin"))

    nt=int(ins.axis(1)[0])
    maxsize=int(self.param("maxsize"))*1000*1000/(30*int(nt))

    nblock=int(self.param("nblock"))
    if not nblock: nblock=ins.axis(2)[0]

    #get the section parameters
    sect_pars=self.build_sect_params(ins.axis(2)[0],nblock)
    self.add_param("program","AMO_prep.x")

    self.add_param("pverb",self.param("verb") )
    self.par_files={}
    self.par_files["intag"]=SEP.pf_irreg.parfile(name=self.param("stdin"), 
      dff_axis=3, tag="<",usage="INPUT",njobs=nblock,
      restart=restart,nblock=nblock)

    if self.logic("collect_out"):
      self.par_files["outtag"]=SEP.pf_split.parfile(name=self.param("stdout"), 
        dff_axis=1, tag=">",usage="OUTPUT",njobs=nblock,
        restart=restart,nblock=nblock)
    else:
      self.par_files["outtag"]=SEP.pf_split.parfile(name=self.param("stdout"), 
        dff_axis=1, tag=">",usage="OUTPUT",njobs=nblock,
        restart=restart,nblock=nblock,remove=None,collect_it=None)


    self.par_files["headers"]=SEP.pf_split.parfile(name=self.param("headers"), 
      dff_axis=2, tag="headers=",usage="OUTPUT",njobs=nblock,
      restart=restart,nblock=nblock)

    self.par_files["vrms"]=SEP.pf_copy.parfile(name=self.param("vrms"), 
      tag="vel=",usage="INPUT",njobs=nblock,
      restart=restart,nblock=nblock)

    self.add_param("files",self.par_files)
    order=[]
    for i in range(nblock): order.append(str(i))
    self.add_param("order",order)
    self.add_param("sect_pars",sect_pars)
    self.add_param("global_pars",self.return_set_params(self.param_list))
    SEP.pj_base.par_job.prep_run(self,restart)
    

  def build_sect_params(self,nf,nblock):
    """Build parameters for parallel job"""
    sect_pars={}

    for i in range(nblock):
      sect_pars[str(i)]=SEP.args.basic(name=str(i))
    return sect_pars
