import SEP.sepfile
import SEP.util
import SEP.spawn
import types,math
import string
import SEP.paths
import SEP.pj_base
import SEP.pf_resplit
import SEP.pf_split
__author__ = "Robert G. Clapp"
__email__ = "bob@sep.stanford.edu"
__version = "2005.03.15"


class  to_time(SEP.pj_base.par_job):
  def __init__(self,name="AMO_to_zero"):
    """Initialize the AMO  program"""
    SEP.pj_base.par_job.__init__(self,name)
    self.add_doc_params()
 
  def add_doc_params(self):
    """Parameters for AMO_to_freq"""
    SEP.pj_base.par_job.add_doc_params(self)

    self.add_doc_param("stdin",doc="Input file")
    self.add_doc_param("stdout",doc="Output file")

    self.add_doc_param("maxsize",1000,doc="Maximum amount of memory to use")

    self.add_doc_param("vrms",doc="VRMS velocity")
    self.add_doc_param("verb","y",doc="Whether or not to be verbose")
    self.add_doc_param("at__n",doc="Number of samples in time")
    self.add_doc_param("at__o",doc="First sample in time")
    self.add_doc_param("nblock",doc="Number of blocks to break along cmpx")
    self.add_doc_param("at__d",doc="Sampling along time axis")
    self.add_doc_param("t_min_log",.2,doc="Minimum time for logstrech")


    self.add_doc_param("distributed_in","n","Whether or not the input is distributed")
    self.param_list=["verb","t_min_log","f_max",
     "vrms","maxsize","at__n","at__o","at__d"]

    self.delete_object_par_params()


  def prep_run(self,restart=None):  
    """Check to make sure the parameters make sense"""

    restart=self.main.param("restart")
    nblock=int(self.param("nblock"))

    #get the section parameters
    sect_pars=self.build_sect_params(nblock)
    self.add_param("program","AMO_to_time.x")
    self.add_param("pverb",self.param("verb") )

    par_files={}
    if  self.logic("distributed_in"):
      par_files["intag"]=SEP.pf_resplit.parfile(name=self.param("stdin"), 
        dff_axis=1, tag="<",usage="INPUT",njobs=nblock,old_file=self.param("old_file"),
        restart=restart,nblock=nblock)
    else:
      par_files["intag"]=SEP.pf_split.parfile(name=self.param("stdin"), 
        dff_axis=1, tag="<",usage="INPUT",njobs=nblock,
         restart=restart,nblock=nblock)


    par_files["outtag"]=SEP.pf_split.parfile(name=self.param("stdout"), 
      dff_axis=2, tag=">",usage="OUTPUT",njobs=nblock,
      restart=restart,nblock=nblock)

    par_files["vrms"]=SEP.pf_copy.parfile(name=self.param("vrms"), 
      tag="vel=",usage="INPUT",njobs=nblock,
      restart=restart,nblock=nblock)

    self.add_param("files",par_files)
    order=[]
    for i in range(nblock): order.append(str(i))
    self.add_param("order",order)
    self.add_param("sect_pars",sect_pars)
    self.add_param("global_pars",self.return_set_params(self.param_list))
    SEP.pj_base.par_job.prep_run(self,restart)
    

  def build_sect_params(self,nblock):
    """Build parameters for parallel job"""
    sect_pars={}

    for i in range(nblock):
      sect_pars[str(i)]=SEP.args.basic(name=str(i))
    return sect_pars
