module amo_time_freq_mod
  use sep3d_struct_mod
  use amo_wrap_mod
  use amo_interp_mod
  use amo_nmo_mod
  use sep
  use verb_mod
  implicit none
  character(len=128),private :: key1,key2,key3,key4
  type(sep3d),private        :: irreg_s,freq_s,reg_s,f_reg
  integer,private,save       :: nf
  logical :: do_nmo

  contains


  logical function time_to_freq_init(d_s,d_pad_s,s_tag,f_bef,headers_s,do2d)
    type(sep3d)       :: d_s,f_bef,vel_s,headers_s,d_pad_s
    character(len=*)  :: s_tag
    logical           :: do2d

    time_to_freq_init=.false.



    !output

    call amo_get_n1_size(d_pad_s,nf,do2d,1)

    !parameters
    call from_param("nmo_it",do_nmo,.true.)
    call from_param("mx_key",key1,"cmp_x")
    call from_param("my_key",key2,"cmp_y")
    call from_param("hx_key",key3,"h_x")
    call from_param("hy_key",key4,"h_y")

    call init_sep3d(f_bef,"OUTPUT","COMPLEX","REGULAR",(/d_s%ntraces,nf/))
    call init_sep3d(headers_s,"OUTPUT","FLOAT","REGULAR",(/4,d_s%ntraces/))
    call amo_nmo_init(d_s,s_tag,100)

    irreg_s=d_s
    freq_s=f_bef

    time_to_freq_init=.true.
  end function




  subroutine to_transp_freq(intag,outtag,locs,verb)
    type(sep3d)            :: no_head
    character(len=*)       :: intag,outtag
    real, allocatable      :: realcube(:,:),real2(:,:)
    complex,allocatable    :: cmplxcube(:,:),bigc(:,:)
    real, allocatable      :: headers(:,:),key(:)
    real,pointer           :: locs(:,:),off(:)
    integer                :: ndone,nblock,nh,i,i_f
    logical                :: verb 

    allocate(realcube(irreg_s%n(1),1000),cmplxcube(nf,1000))
    allocate(real2(irreg_s%n(1),1000))
    ndone=0;
    allocate(headers(4,1000),key(1000))
    if(.not.init_print_progress(0,irreg_s%ntraces,verb)) &
      call seperr("trouble initialzing verbosity")
    call getch_add_string("in.ignore_gff=1")
    call init_sep3d(intag,no_head,"INPUT")
    allocate(locs(4,irreg_s%ntraces))
    if(irreg_s%ntraces==0) return
    do while (ndone < irreg_s%ntraces)
      nblock=min(1000,irreg_s%ntraces-ndone)
      if(nblock/=1000) then
         deallocate(realcube,real2,cmplxcube,headers)
         allocate(realcube(irreg_s%n(1),nblock),cmplxcube(nf,nblock))
         allocate(real2(irreg_s%n(1),nblock),headers(4,nblock))
      end if 


      call print_progress("transforming to frequency",nblock)

      call sep3d_grab_headers(intag,no_head,nh,nwind=(/nblock/),fwind=(/ndone/))



      call sep3d_grab_key_vals(no_head,trim(key1),key); 
      headers(1,:)=key
      call sep3d_grab_key_vals(no_head,trim(key2),key); 
      headers(2,:)=key
      call sep3d_grab_key_vals(no_head,trim(key3),key); 
      headers(3,:)=key
      call sep3d_grab_key_vals(no_head,trim(key4),key); 
      headers(4,:)=key
      allocate(off(size(headers,2)))
      off=2*(sqrt(headers(3,:)**2+headers(4,:)**2))

      if(.not. sep3d_read_data("in",no_head,real2)) call seperr("trouble reading data")

      !nmo 
      if(do_nmo) then
        call nmoit(real2,realcube,headers(1,:),headers(2,:),off)
      else
        realcube=real2
      end if

      !amo transform
      if(.not. amo_transform_to_mspace(nblock,realcube,cmplxcube)) call seperr("trouble stretching")

      !store the headers
      do i=1,nblock
        locs(1,ndone+i)=headers(1,i)
        locs(2,ndone+i)=headers(2,i)
        locs(3,ndone+i)=headers(3,i)
        locs(4,ndone+i)=headers(4,i)
      end do

      call srite(outtag,cmplxcube,nblock*nf*8)
      ndone=ndone+nblock
    end do
    deallocate(realcube,real2,cmplxcube)

  end subroutine


  logical function freq_to_time_init(s_tag,freq_reg,r_s,nt,ot,dt,init_nmo,init_amo)
    type(sep3d)       :: f_bef,vel_s,r_s,freq_reg
    character(len=*)  :: s_tag
    integer           :: nf,n(5),nt
    real              :: o(5),d(5),ot,dt
    logical           :: init_nmo,init_amo

    freq_to_time_init=.false.
    call from_param("nmo_it",do_nmo,.true.)
    n(1)=nt; o(1)=ot; d(1)=dt 
    f_reg=freq_reg
    n(2:5)=freq_reg%n(1:4)
    o(2:5)=freq_reg%o(1:4)
    d(2:5)=freq_reg%d(1:4)
    call init_sep3d(r_s,"OUTPUT","FLOAT","REGULAR",n,o,d)
    reg_s=r_s
    nf=freq_reg%n(5)
    if(init_amo)call amo_get_n1_size(r_s,nf,.false.,0)
    if(init_nmo) call amo_nmo_init(r_s,s_tag,100)
    freq_to_time_init=.true.
  end function


  subroutine from_transp_freq(intag,outtag,verb,p2,p3)
    character(len=*)       :: intag,outtag
          real, allocatable      :: realcube(:,:),real2(:,:),x(:,:),y(:,:),off(:)
    complex,allocatable    :: cmplxcube(:,:),bigc(:,:,:)
    real, allocatable      :: headers(:,:),key(:)
    integer                :: ndone,nblock,nh,i,i_f,i2,i3,i4,i5
    logical                :: verb,pad2,pad3
    logical, optional      :: p2,p3
    real :: hx,hy

   pad2=.false.
   pad3=.false.
   if(present(p2)) pad3=p3
   if(present(p3)) pad2=p2

    !transpose dataset
    nf=f_reg%n(5)
    allocate(bigc(nf,reg_s%n(2),reg_s%n(3)))

    !form the offset and location arrays
    allocate(x(reg_s%n(2),reg_s%n(3)),y(reg_s%n(2),reg_s%n(3)),off(reg_s%n(2)))
    do i3=1,reg_s%n(3)
      do i2=1,reg_s%n(2)
        x(i2,i3)=reg_s%o(2)+reg_s%d(2)*(i2-1)
        y(i2,i3)=reg_s%o(3)+reg_s%d(3)*(i3-1)
      end do
    end do

    allocate(realcube(reg_s%n(1),reg_s%n(2)),cmplxcube(nf,reg_s%n(2)))
    allocate(real2(reg_s%n(1),reg_s%n(2)))

    if(.not.init_print_progress(0,reg_s%n(5)*reg_s%n(4)*reg_s%n(3),verb)) &
      call seperr("trouble initialzing verbosity")

    i=0
   do i5=1,reg_s%n(5)
      hy=(i5-1)*reg_s%d(5)+reg_s%o(5)
      do i4=1,reg_s%n(4)
        hx=(i4-1)*reg_s%d(4)+reg_s%o(4)
        call sreed(intag,bigc,size(bigc)*8)
        off=2*sqrt(hx**2+hy**2)
        do i3=1,reg_s%n(3)
          call print_progress("transforming back to time")
          if(.not. pad3 .or. i3 /= reg_s%n(3)) then
            cmplxcube=bigc(:,:,i3)
  
             !amo transform
             if(.not. amo_transform_from_mspace(reg_s%n(2),realcube,cmplxcube)) call seperr("trouble stretching")

             !nmo 
             if(do_nmo) then
               call undo_nmoit(realcube,real2,x(:,i3),y(:,i3),off)
             else 
               real2=realcube
             end if
             if(pad2) call srite(outtag,real2(:,:reg_s%n(2)-1),reg_s%n(1)*(reg_s%n(2)-1)*4)
             if(.not. pad2) call srite(outtag,real2,size(real2)*4)
           end if
           i=i+reg_s%n(2)
        end do
      end do
    end do
    deallocate(x,y,off,realcube,cmplxcube,real2,bigc)
  end subroutine

end module
