import SEP.sepfile
import SEP.util
import SEP.spawn
import types,math
import string
import SEP.paths
import SEP.pf_resplit
import SEP.pf_copy
import SEP.pf_split
import SEP.pj_base
__author__ = "Robert G. Clapp"
__email__ = "bob@sep.stanford.edu"
__version = "2005.03.18"


class  to_zero(SEP.pj_base.par_job):
  def __init__(self,name="AMO_to_zero"):
    """Initialize the AMO  program"""
    SEP.pj_base.par_job.__init__(self,name)
    self.add_doc_params()
 
  def add_doc_params(self):
    """Parameters for AMO_to_freq"""
    SEP.pj_base.par_job.add_doc_params(self)

    self.add_doc_param("stdin",doc="Input file")
    self.add_doc_param("stdout",doc="Output file")

    self.add_doc_param("maxsize",1000,doc="Maximum amount of memory to use")

    self.add_doc_param("verb","y",doc="Whether or not to be verbose")
    self.add_doc_param("amx_n",doc="Number of samples along midpoint x")
    self.add_doc_param("amy_n",doc="Number of samples along midpoint y")
    self.add_doc_param("ahx_n_o",doc="Number of samples along half-offset x of output",required=None)
    self.add_doc_param("ahx_n",doc="Number of samples along half-offset x")
    self.add_doc_param("ahy_n",doc="Number of samples along half-offset y")
    self.add_doc_param("at__n",doc="Number of samples in time")
                                                                                
    self.add_doc_param("amx_o",doc="First sample along midpoint x")
    self.add_doc_param("amy_o",doc="First sample along midpoint y")
    self.add_doc_param("ahx_o",doc="First sample along half-offset x")
    self.add_doc_param("ahx_o_o",doc="First sample along half-offset x of output",required=None)
    self.add_doc_param("ahy_o",doc="First sample along half-offset y")
    self.add_doc_param("at__o",doc="First sample in time")
                                                                                
    self.add_doc_param("amx_d",doc="Sampling along midpoint x")
    self.add_doc_param("amy_d",doc="Sampling along midpoint y")
    self.add_doc_param("ahx_d",doc="Sampling along half-offset x")
    self.add_doc_param("ahx_d_o",doc="Sampling along half-offset y of output",required=None)
    self.add_doc_param("ahy_d",doc="Sampling along half-offset y")
    self.add_doc_param("at__d",doc="Sampling along time axis")

    self.add_doc_param("hx_dist",doc="Distance (in samples) to sum over for AMO transform (defaults to hx*8)",required=None)
    self.add_doc_param("eps",.000001,"Epsilon for normalization calculation")
    self.add_doc_param("normalize","n",doc="Whether or not to normalize")
    self.add_doc_param("f_max",60.,doc="Maximum frequency (60.)")
    self.add_doc_param("t_min_log",doc="Minimum time for log transform",required=None)
    self.add_doc_param("nblock",doc="Number of blocks to break along cmpx")
    self.add_doc_param("distributed_in","n","Whether or not the input is distributed")
    self.add_doc_param("collect_out","y","Whether or not to collect the output")
    self.param_list=["verb","nblock",
     "hx_dist","eps","normalize","f_max","t_min_log"]

    self.param_list.extend(["amx_n","amx_o","amx_d"])
    self.param_list.extend(["amy_n","amy_o","amy_d"])
    self.param_list.extend(["ahx_n_o","ahx_o_o","ahx_d_o"])
    self.param_list.extend(["ahx_n","ahx_o","ahx_d"])
    self.param_list.extend(["ahy_n","ahy_o","ahy_d"])
    self.param_list.extend(["at__n","at__o","at__d"])
    self.delete_object_par_params()


  def prep_run(self,restart=None):  
    """Check to make sure the parameters make sense"""

    restart=self.main.param("restart")

    nblock=int(self.param("nblock"))

    if math.fmod(int(self.param("amy_n")),2) != 0:
        SEP.util.err("All axes need to be even in length axis[amy]=%d"%(int(self.param("amy_n"))))
    if math.fmod(int(self.param("amx_n")),2) != 0:
        SEP.util.err("All axes need to be even in length axis[amx]=%d"%(int(self.param("amx_n"))))

    hx_dist=self.param("hx_dist",error=None)
    if not hx_dist:  hx_dixt=float(self.param("ahx_d"))*8
    self.add_param("hx_dist",hx_dist)

    #get the section parameters
    sect_pars=self.build_sect_params(nblock)
    self.add_param("program","AMO_to_zero.x")

    self.add_param("pverb",self.param("verb") )
    self.par_files={}

    if self.logic("distributed_in"): 
      self.par_files["intag"]=SEP.pf_resplit.parfile(name=self.param("stdin"), 
        dff_axis=2, tag="<",usage="INPUT",njobs=nblock,old_file=self.param("old_file"),
       restart=restart,nblock=nblock)
    else:
      self.par_files["intag"]=SEP.pf_split.parfile(name=self.param("stdin"), 
        dff_axis=2, tag="<",usage="INPUT",njobs=nblock,
       restart=restart,nblock=nblock)

    if self.logic("collect_out"):
      self.par_files["outtag"]=SEP.pf_split.parfile(name=self.param("stdout"), 
        dff_axis=5, tag=">",usage="OUTPUT",njobs=nblock,
        restart=restart,nblock=nblock)
    else:
      self.par_files["outtag"]=SEP.pf_split.parfile(name=self.param("stdout"), 
        dff_axis=5, tag=">",usage="OUTPUT",njobs=nblock,
        restart=restart,nblock=nblock,remove=None,collect_it=None)


    self.par_files["headers"]=SEP.pf_copy.parfile(name=self.param("headers"), 
      tag="headers=",usage="INPUT",njobs=nblock,
      restart=restart,nblock=nblock)

    self.add_param("files",self.par_files)
    order=[]
    for i in range(nblock): order.append(str(i))
    self.add_param("order",order)
    self.add_param("sect_pars",sect_pars)
    self.add_param("global_pars",self.return_set_params(self.param_list))
    SEP.pj_base.par_job.prep_run(self,restart)
    

  def build_sect_params(self,nblock):
    """Build parameters for parallel job"""
    sect_pars={}

    for i in range(nblock):
      sect_pars[str(i)]=SEP.args.basic(name=str(i))
    return sect_pars
