module axis_type_mod


 type axis_type
   real, pointer :: scaleb(:),scalee(:)
   integer       :: ib_d0,ib_d1,ie_d0,ie_d1
   integer       :: ib_m0,ib_m1,ie_m0,ie_m1
   logical       :: beg,end
 end type
 

end module
module patch_oc_mod
  use sep
  use sep3d_struct_mod
  use helixcartmod
  use axis_type_mod
  implicit none
  type(sep3d) :: model,data
  type(axis_type) , pointer :: axes(:)


  contains

  logical function init_patching(model_s,data_s)
    type(sep3d) :: data_s,model_s
    integer :: i,nd,iaxes,maxsize
    integer :: ibeg0(9),ibeg1(9),iend1(9),iend0(9)
    logical :: not_default(9)
    character(len=128) :: temp_ch


    init_patching=.false.
    data=data_s
    model=model_s

    allocate(axes(data_s%ndims))
    do i=1,data_s%ndims
      not_default(i)=.false.
      
      temp_ch="beg0_"; call parcat(temp_ch,i);
      call from_param(temp_ch,ibeg0(i),1)
      if(ibeg0(i)/=1) not_default(i)=.true.
    
      temp_ch="beg1_"; call parcat(temp_ch,i);
      call from_param(temp_ch,ibeg1(i),1)
    
      temp_ch="end1_"; call parcat(temp_ch,i);
      call from_param(temp_ch,iend1(i),data_s%n(i))

      temp_ch="end0_"; call parcat(temp_ch,i);
      call from_param(temp_ch,iend0(i),data_s%n(i))

      if(.not. setup_axis(model_s%n(i),axes(i),ibeg0(i),ibeg1(i),&
        iend1(i),iend0(i))) then
        write(0,*) "trouble setting up axis=",i
        return
      end if
      if(model_s%ndims >= i) then
      if(model_s%n(i) >0 ) then
      if(iend0(i)-ibeg0(i)+1 /= model_s%n(i)) then
        write(0,*) ibeg0(i),"=ibeg0 iend0=",iend0(i)
        write(0,*) model_s%n(i),"=n patch size not equal to model size axis=",i
        return
      end if
      end if
      end if
    end do
    call from_param("maxsize",maxsize,100)
    maxsize=100*1000*125
    if(model_s%data_format(1:1)=="C") maxsize=maxsize/2
    call init_loop_calc(model_s%ndims,model_s%n,"PATCH LOOP",maxsize)
    model=model_s
    init_patching=.true.
  end function


  subroutine patch_it_c(adj,add,mtag,dtag)
    logical :: adj,add
    character(len=*)     :: mtag,dtag
    integer, pointer     :: nmodel(:),fmodel(:),jmodel(:)
    integer, pointer     :: ndata(:),fdata(:),jdata(:)
    integer, pointer     :: iloc(:)
    real, allocatable :: cmodel(:,:),cdata(:,:)
    logical              :: first
    integer,external     :: sreed_window,srite_window,do_sep_loop
    integer              :: esize
    integer              :: i,iax,ierr,ndims
    

    first=.true.

    esize=4
 ndims=5

  
    allocate(ndata(ndims),fdata(ndims),jdata(ndims))
    allocate(nmodel(ndims),fmodel(ndims),jmodel(ndims))
    allocate(iloc(ndims))
    jmodel=1;jdata=1
    

    ierr=do_sep_loop("PATCH LOOP",nmodel,fmodel)
    do while(ierr==0) 
      !ALLOCATE MEMORY 
      if(first) then
        if(nmodel(1)/=model%n(1)) &
          call seperr("Need to be able to hold first axis in memory")
        allocate(cmodel(nmodel(1),product(nmodel(2:))))
        allocate(cdata(nmodel(1),product(nmodel(2:))))
        first=.false.
      end if 
      ndata=nmodel
      do iax=1,size(nmodel)
        fdata(iax)=fmodel(iax)+axes(iax)%ib_d0-1
      end do
      !READ IN DATASETS
      if(adj .or.  add) then
        if(0/=sreed_window(dtag,ndims,data%n,ndata,fdata,jdata, &
          esize,cdata)) then
          write(0,*) "trouble in reading data window"
          write(0,*) "DATA SIZE",data%n
          write(0,*) "nwind",ndata
          write(0,*) "fwind",fdata
          call seperr("") 
        end if
      else 
        cdata=cmplx(0.,0.)
      end if
      if(.not. adj .or.  add) then
        if(0/=sreed_window(mtag,ndims,model%n,nmodel,fmodel,jmodel, &
          esize,cmodel)) then
          write(0,*) "trouble in reading model window",trim(mtag)
          call seperr("") 
        end if
      else 
        cmodel=cmplx(0.,0.)
      end if

      !APPLY PATCH
      do i=1,product(nmodel(2:))
        !scale the first axis if necessary
        if(axes(1)%beg) then
          if(.not. adj) then
            cmodel(axes(1)%ib_m0:axes(1)%ib_m1,i)=&
              cmodel(axes(1)%ib_m0:axes(1)%ib_m1,i)*axes(1)%scaleb
          else
            cdata(axes(1)%ib_m0:axes(1)%ib_m1,i)=&
              cdata(axes(1)%ib_m0:axes(1)%ib_m1,i)*axes(1)%scaleb
          end if
        end if
        if(axes(1)%end) then
          if(.not. adj) then
            cmodel(axes(1)%ie_m1:axes(1)%ie_m0,i)=&
              cmodel(axes(1)%ie_m1:axes(1)%ie_m0,i)*axes(1)%scalee
          else
            cdata(axes(1)%ie_m1:axes(1)%ie_m0,i)=&
              cdata(axes(1)%ie_m1:axes(1)%ie_m0,i)*axes(1)%scalee
          end if
        end if
        call  helix2cart(model%n(2:),i,iloc(2:))

        do iax=2,size(nmodel)
          if(iloc(iax) <= axes(iax)%ib_m1) then
            if(.not. adj) then
              cmodel(:,i)=cmodel(:,i)*axes(iax)%scaleb(iloc(iax))
            else 
              cdata(:,i)=cdata(:,i)*axes(iax)%scaleb(iloc(iax))
            end if
          end if
          if(iloc(iax) >= axes(iax)%ie_m1) then
            if(.not. adj) then
              cmodel(:,i)=cmodel(:,i)*axes(iax)%scalee(iloc(iax)-&
               axes(iax)%ie_m1+1)
            else 
              cdata(:,i)=cdata(:,i)*axes(iax)%scalee(iloc(iax)-&
               axes(iax)%ie_m1+1)
            end if
          end if
        end do
      end do

write(0,*) "CHECK THIS",cdata(1,1),cmodel(1,1)
      if(.not. adj)  cdata=cdata+cmodel
      if(adj)  cmodel=cmodel+cdata
      !WRITE OUT DATA
      if(.not. adj ) then
        if(0/=srite_window(dtag,ndims,data%n,ndata,fdata,jdata, &
          esize,cdata)) then
          write(0,*) "trouble in reading data window",trim(dtag)
          call seperr("") 
        end if
      end if
      if(adj ) then
        if(0/=srite_window(mtag,ndims,model%n,nmodel,fmodel,jmodel, &
          esize,cmodel)) then
          write(0,*) "trouble in reading model window",trim(mtag)
          call seperr("") 
        end if
      end if
      ierr=do_sep_loop("PATCH LOOP",nmodel,fmodel)
    end do
    deallocate(ndata,fdata,jdata,iloc)
    deallocate(nmodel,fmodel,jmodel)
    deallocate(cmodel,cdata)
  end subroutine 




  logical function setup_axis(n,ax,ibeg0,ibeg1,iend1,iend0)
    integer :: ibeg0,iend0,ibeg1,iend1
    type(axis_type) :: ax
    integer         :: i,n

    setup_axis=.false.

    allocate(ax%scaleb(ibeg1-ibeg0+1))
    allocate(ax%scalee(iend0-iend1+1))

    ax%ib_m0=1
    ax%ib_m1=ibeg1-ibeg0+1
    ax%ie_m1=n-(iend0-iend1)+1
    ax%ie_m0=n

    ax%ib_d0=ibeg0
    ax%ib_d1=ibeg1
    ax%ie_d0=iend0
    ax%ie_d1=iend1

    ax%scalee=1
    ax%scaleb=1
    ax%end=.false.
    ax%beg=.false.
    if(iend0 < iend1 .or. ibeg1 < ibeg0 .or. ibeg1 > iend1) then
       write(0,*) "beg end problem "
       write(0,*)  ibeg0,"=ibeg0 ibeg1=",ibeg1
       write(0,*)  iend0,"=iend0 iend1=",iend1
       return
     end if
 
    if(ibeg1 /= ibeg0) then
      ax%beg=.true.
      do i=1,ibeg1-ibeg0+1
        ax%scaleb(i)=real(i-1)/real(ibeg1-ibeg0)      
      end do 
    end if
    if(iend1 /= iend0) then
      ax%end=.true.
      do i=1,iend0-iend1+1
        ax%scalee(i)=1.-real(i-1)/real(iend0-iend1)      
      end do 
    end if
    setup_axis=.true.
  end function


end module
