!$
!$=head1 NAME
!$
!$polydiv - polynomial division
!$
!$=head1 SYNOPSIS
!$
!$Initializer - C<call polydiv_init(nd,aa)>
!$
!$Operator    - C<ierr=polydiv_lop(adj,add,xx,yy)>
!$
!$=head1 PARAMETERS
!$
!$=over 4
!$
!$=item nd - integer
!$
!$      number of data points
!$
!$=item aa - type(filter)
!$
!$      helix filter to perform convolution with
!$
!$=item adj,add,xx,yy -
!$
!$      standard operators parameters
!$
!$=back
!$
!$=head1 DESCRIPTION
!$
!$ Polynomial division (deconvolution), inverse to convolution.
!$ Requires the filter be causal with an implicit "1." at the onset.
!$
!$
!$=head1 SEE ALSO
!$
!$L<helix>,L<hconest>,L<helicon>,L<npolydiv>
!$
!$=head1 LIBRARY
!$
!$B<geef90>
!$
!$=cut
module polydiv_func_mod
  use amo_wrap_mod
! Helix polynomial division
  use helix
  implicit none
  integer, private :: n1,n2,n3
  type( filter), private :: aa 
  complex ,private,  dimension (:,:,:), allocatable  :: tt 
  complex ,private,  dimension (:,:), allocatable  :: buf 
  contains
  subroutine polydiv_func_init ( n1_in,n2_in,n3_in,aa_in )
    integer    :: n1_in ,n2_in,n3_in
    type( filter)    :: aa_in 
    n1 = n1_in 
    n2 = n2_in 
    n3 = n3_in 
    aa = aa_in 
    if (.not. allocated(tt)) then
      allocate(tt ( n1,n2,n3)) 
      allocate(buf ( n1,n2)) 
    end if
  end subroutine
  subroutine polydiv_func_lop2(adj,xx,yy)
    logical,intent(in) :: adj
    integer :: ierr
    complex, dimension (:,:,:)  :: xx 
    complex, dimension (:,:,:)  :: yy 
    integer  ia, ix, iy
    tt = 0.
    if ( adj) then
      do ix= n3, 1, -1  
        tt(:,:, ix) = yy(:,:, ix)
        do ia = 1, size( aa%lag)
          iy = ix + aa%lag( ia)
          if ( iy > n3) then
            cycle
          end if
          buf=tt(:,:,iy)
          ierr=transform_tt(iy,ix,buf)
          tt( :,:,ix) =        tt(:,:, ix) -  aa%flt( ia) * buf
        end do
      end do 
      xx =        xx + tt
    else
      do iy= 1, n3  
        tt(:,:, iy) = xx(:, :,iy)
        do ia = 1, size( aa%lag)
          ix = iy - aa%lag( ia)
          if ( ix < 1) then
            cycle
          end if
          buf=tt(:,:,ix)
          ierr=transform_tt(ix,iy,buf)
          tt( :,:,iy) =  tt(:,:, iy) -  aa%flt( ia) * buf
        end do
      end do 
      yy =        yy + tt
    end if
  end subroutine 
  subroutine polydiv_func_close()
    deallocate( tt,buf )
  end subroutine 
end module 
